"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var helpTopic_1 = require("../helpTopic");
var VolunteerCommand = /** @class */ (function () {
    function VolunteerCommand(volunteer) {
        this.name = 'volunteer';
        this.description = 'shows volunteer experience';
        this.volunteer = volunteer;
        this.helpTopic = new helpTopic_1.HelpTopic(this, {
            synopsis: 'volunteer [--currentYear XOR -y]',
            options: {
                '--currentYear': 'List all volunteer experience in the current year',
                '-y': 'List all volunteer experience in which the year -y is between startYear and endYear',
            },
            examples: [
                {
                    cmd: 'volunteer --currentYear',
                    description: 'The following command shows all volunteer experience that happened in the current year'
                },
                {
                    cmd: 'volunteer -y 2018',
                    description: 'The following command shows all volunteer experience that happened in the 2018 year'
                },
                {
                    cmd: 'volunteer --currentYear -y 2017',
                    description: 'The following command shows all volunteer experience that happened in the current year (Because --currentYear has more precedence than -y)'
                }
            ]
        });
    }
    VolunteerCommand.prototype.run = function (args) {
        var data = this.volunteer;
        if (args.currentYear) {
            var currentYear_1 = new Date().getUTCFullYear() - 1;
            data = data.filter(function (v) { return v.startYear <= currentYear_1 && v.endYear >= currentYear_1; });
        }
        else if (args.y) {
            data = data.filter(function (v) { return v.startYear <= args.y && v.endYear >= args.y; });
        }
        return data;
    };
    return VolunteerCommand;
}());
exports.VolunteerCommand = VolunteerCommand;
//# sourceMappingURL=VolunteerCommand.js.map