"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var helpTopic_1 = require("../helpTopic");
var WorkCommand = /** @class */ (function () {
    function WorkCommand(work) {
        this.name = 'work';
        this.description = 'shows work experience';
        this.work = work;
        this.helpTopic = new helpTopic_1.HelpTopic(this, {
            synopsis: 'work [--current XOR -y]',
            options: {
                '--current': 'List all work experience in the current year',
                '-y': 'List all work experience in which the year -y is between startYear and endYear',
            },
            examples: [
                {
                    cmd: 'work --current',
                    description: 'The following command shows the current work experience'
                },
                {
                    cmd: 'work -y 2018',
                    description: 'The following command shows all the work experience that happened in the 2018 year'
                },
                {
                    cmd: 'work --current -y 2017',
                    description: 'The following command shows the current work experience (Because --current has more precedence than -y)'
                }
            ]
        });
    }
    WorkCommand.prototype.run = function (args) {
        var data = this.work;
        if (args.current) {
            data = data.filter(function (w) { return w.endYear === null; });
        }
        else if (args.y) {
            data = data.filter(function (w) { return w.startYear <= args.y && w.endYear >= args.y; });
        }
        return data;
    };
    return WorkCommand;
}());
exports.WorkCommand = WorkCommand;
//# sourceMappingURL=WorkCommand.js.map