"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var cli_table2_1 = __importDefault(require("cli-table2"));
var easy_table_1 = __importDefault(require("easy-table"));
var events_1 = require("events");
var minimist_1 = __importDefault(require("minimist"));
var xterm_1 = require("xterm");
var fit_1 = require("xterm/lib/addons/fit/fit");
var webLinks_1 = require("xterm/lib/addons/webLinks/webLinks");
var EOL = '\r\n';
var Cli = /** @class */ (function (_super) {
    __extends(Cli, _super);
    function Cli() {
        var _this = _super.call(this) || this;
        _this.commands = {};
        _this.helpTopics = {};
        _this.buffer = '';
        _this.cursorOffset = 0;
        _this.terminalPrompt = 'bashme$ ';
        _this.terminalHistory = [];
        _this.terminalHistoryIndex = 0;
        _this.terminal = new xterm_1.Terminal({
            convertEol: true,
            cursorBlink: true,
            cursorStyle: 'underline',
            fontFamily: 'Menlo, "DejaVu Sans Mono", Consolas, "Lucida Console", monospace',
            fontSize: 12
        });
        _this.terminal.on('key', function (d) { return _this.input(d); });
        _this.terminal.on('paste', function (d) { return _this.input(d); });
        return _this;
    }
    Cli.prototype.input = function (str) {
        for (var i = 0; i < str.length; i++) {
            //console.log(str.charCodeAt(i))
            switch (str.charCodeAt(i)) {
                case 8: // backspace
                    if (this.cursorOffset === 0)
                        break;
                    this.cursorOffset--;
                    this.buffer = this.buffer.substr(0, this.cursorOffset) + this.buffer.substr(this.cursorOffset + 1);
                    this.terminal.write('\b \b');
                    break;
                case 127: // del
                    if (this.cursorOffset === 0)
                        break;
                    // TODO: change it so it works outside browser too
                    if (navigator.platform.indexOf('Mac') >= 0) {
                        this.cursorOffset--;
                        this.buffer = this.buffer.substr(0, this.cursorOffset) + this.buffer.substr(this.cursorOffset + 1);
                        this.terminal.write('\b \b');
                    }
                    break;
                case 13: // enter
                    this.processInput();
                    this.prompt(this.terminalHistory[this.terminalHistoryIndex - 1] !== 'clear');
                    break;
                case 65: // up
                    var cmd = this.terminalHistory[this.terminalHistoryIndex - 1];
                    if (cmd) {
                        this.terminalHistoryIndex--;
                        this.terminal.write(Array(this.buffer.length + 1).join('\b \b'));
                        this.terminal.write(cmd);
                        this.cursorOffset = cmd.length;
                        this.buffer = cmd;
                    }
                    break;
                case 66: // down
                    var cmd = this.terminalHistory[this.terminalHistoryIndex + 1];
                    if (cmd) {
                        this.terminalHistoryIndex++;
                        this.terminal.write(Array(this.buffer.length + 1).join('\b \b'));
                        this.terminal.write(cmd);
                        this.cursorOffset = cmd.length;
                        this.buffer = cmd;
                    }
                    break;
                case 67: // right
                case 68: // left
                    if (str.charCodeAt(i) === 67)
                        this.cursorOffset++;
                    if (str.charCodeAt(i) === 68)
                        this.cursorOffset--;
                    this.terminal.write(String.fromCharCode(27) + String.fromCharCode(91) + str[i]);
                    break;
                case 27:
                case 91:
                    // skip
                    break;
                default:
                    var rest = this.buffer.substr(this.cursorOffset);
                    this.buffer = this.buffer.substr(0, this.cursorOffset) + str[i] + rest;
                    this.cursorOffset++;
                    this.terminal.write(str[i] + rest + Array(rest.length + 1).join('\b'));
                    break;
            }
        }
    };
    Cli.prototype.processInput = function () {
        var buffer = this.buffer.trim();
        if (buffer) {
            var args = minimist_1.default(buffer.split(' '));
            var cmdName = args._[0];
            this.emit('input', cmdName, args);
            this.processCommand(cmdName, args);
            this.terminalHistory.push(buffer);
            this.terminalHistoryIndex = this.terminalHistory.length;
        }
        this.cursorOffset = 0;
        this.buffer = '';
    };
    Cli.prototype.processCommand = function (cmdName, args) {
        if (cmdName === 'clear') {
            this.terminal.reset();
        }
        else if (cmdName === 'help') {
            args._[1] ? this.showHelpTopic(args._[1]) : this.showHelp();
        }
        else if (this.commands[cmdName]) {
            var output = this.commands[cmdName].run(args);
            this.write("" + EOL + this.processCommandOutput(output));
        }
        else {
            this.write("" + EOL + cmdName + ": command not found");
        }
    };
    Cli.prototype.processCommandOutput = function (output) {
        if (typeof output === 'object') {
            if (Array.isArray(output)) {
                // print as table
                if (!output.length)
                    return '';
                var cols = Object.keys(output[0]).length;
                var colWidths = Array(cols).fill(Math.floor(this.terminal.cols / cols - 1));
                colWidths[colWidths.length - 1] += this.terminal.cols % cols - 1;
                var table_1 = new cli_table2_1.default({
                    colWidths: colWidths,
                    head: Object.keys(output[0]),
                    wordWrap: true,
                });
                output.forEach(function (item) {
                    // @ts-ignore
                    table_1.push(Object.values(item));
                });
                return table_1.toString();
            }
            else {
                // print as json
                return JSON.stringify(output, null, 2);
            }
        }
        else {
            return output.toString();
        }
    };
    Cli.prototype.prompt = function (newLine) {
        if (newLine === void 0) { newLine = true; }
        this.terminal.write("" + (newLine ? EOL : '') + this.terminalPrompt);
    };
    Cli.prototype.showHelp = function () {
        var table = new easy_table_1.default();
        table.separator = '\t\t';
        for (var i in this.commands) {
            table.cell('name', this.commands[i].name);
            table.cell('description', this.commands[i].description);
            table.newRow();
        }
        this.terminal.write("" + EOL + EOL + "These commands are defined." + EOL + "Type `help name` to find out more about the function `name`." + EOL + EOL + "\t" + table.print().replace(/\r?\n/g, EOL + "\t"));
    };
    Cli.prototype.showHelpTopic = function (cmdName) {
        if (this.helpTopics[cmdName]) {
            this.write("" + EOL + EOL + this.helpTopics[cmdName] + EOL);
        }
        else {
            this.write(EOL + "no help topics match `" + cmdName + "`");
        }
    };
    Cli.prototype.register = function (command) {
        this.commands[command.name] = command;
        if (command.helpTopic) {
            this.helpTopics[command.name] = command.helpTopic;
        }
    };
    Cli.prototype.show = function (domElement) {
        var _this = this;
        this.terminal.open(domElement);
        this.terminal.focus();
        this.prompt(false);
        fit_1.fit(this.terminal);
        webLinks_1.webLinksInit(this.terminal);
        window.onresize = function () { return fit_1.fit(_this.terminal); };
    };
    Cli.prototype.write = function (str) {
        this.terminal.write(str);
    };
    return Cli;
}(events_1.EventEmitter));
exports.Cli = Cli;
//# sourceMappingURL=index.js.map